using NLOptControl

# =========== PROBLEM DEFINITION ==============================
# Constraints on states
XL   = [-Inf,-Inf,-Inf,-Inf]
XU   = [Inf,Inf,Inf,Inf]
# Constraints on control, we will add ux^2+uy^2 <= 1, so these won't be active
CL   = [-1.0, -1.0]
CU   = [1.0, 1.0]
# Initial and final constraints on states
X0   = [-1.5, 0, 0, 0]
XF   = [1.5, 0, 0, 0]

# Create problem ===============================================================
prob  = define(numStates = 4, numControls=2,
    X0=X0, XF=XF, CL=CL, CU=CU, XL=XL, XU=XU)
# Give name to states and controls
states!(prob, [:x, :vx, :y, :vy],
    descriptions=["x(t)", "vx(t)", "y(t)", "vy(t))"])

controls!(prob, [:ux, :uy], descriptions=["ux", "uy"])

# Setup model equations as julia **expressions** ===============================
dx    = Array{Expr}(4)
dx[1] = :( vx[j] )
dx[2] = :( ux[j] )
dx[3] = :( vy[j] )
dx[4] = :( uy[j] )

## Add dynamics to model =======================================================
dynamics!(prob, dx)
