using Ipopt         # Solver
using NLOptControl  # Problem definition and discretization
using PrettyPlots   # Plots for optimization problem
using Plots         # Custom plots

include("DoubleIntegrator2D.jl")
# The problem is now defined as "prob"

# Obstracle constraints
expr1 = :( ux[j]^2 + uy[j]^2 <= 1 )
expr2 = :( y[j] >= cos(x[j]) - 0.2 )

# Try with both [expr1] and [expr1, expr2]
constraints!(prob, [expr1])

# Number of grid points in discretization
N = 700

# Final time fixed, backward Euler discretization
configure!(prob; finalTimeDV=false, tf=4.5, integrationScheme=:bkwEuler, N=N)

# Set objective
obj = integrate!(prob, :( ux[j]^2 + uy[j]^2) )
# Set to minimize obj
@NLobjective(prob.ocp.mdl, Min, obj)

@time optimize!(prob)

# This plot from "PrettyPlots" plots all states
# allPlots(prob)

# Get the internal data
data = prob.r.ocp.dfs[1]

# Plot x, vx, ux
plot(data[:t], data[:x], lab="x(t)", xlab="t")
plot!(data[:t], data[:vx], lab="vx(t)")
plot!(data[:t], data[:ux], lab="ux(t)")

# Plot x,y plane
plot(data[:x], data[:y], xlims=(-1.5,1.5), ylims=(-1.5, 1.5),
    label="trajectory", xlabel="x", ylabel="y", reuse=false)
    # reuse=false means use new plot window

# Plot constraint
xs = -1.5:0.1:1.5
plot!(xs, cos.(xs)-0.2, label="constraint")
